@@ -26,7 +26,7 @@ import argparse
 import shlex
 import os
 import subprocess
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk, GLib, Gio
 
 import psutil
 old_psutil_format = isinstance(psutil.Process.username, property)
@@ -48,17 +48,8 @@ class LightLockerSettings:
 
     def __init__(self):
         '''Initialize the Light Locker Settings application.'''
-        self.systemwide_desktop_file = \
-            "/etc/xdg/autostart/light-locker.desktop"
-        self.user_desktop_file = os.path.join(os.path.expanduser('~'),
-                                              '.config',
-                                              'autostart',
-                                              'light-locker.desktop')
-        self.screenblank_timeout_desktop_file = os.path.join(
-            os.path.expanduser('~'),
-            '.config',
-            'autostart',
-            'screensaver-settings.desktop')
+        self.light_locker_keyfile = None
+        self.screensaver_keyfile = None
 
         self.builder = Gtk.Builder()
         self.builder.set_translation_domain ('light-locker-settings')
@@ -102,6 +93,8 @@ class LightLockerSettings:
 
         self.screensaver_managed = False
 
+        self.gsettings_init()
+
         self.init_settings()
 
         ''' Monitor changes to the settings '''
@@ -180,12 +173,6 @@ class LightLockerSettings:
 
     def apply_cb(self, button, data=None):
         '''Apply changes and update the relevant setting files.'''
-        ''' Check whether the autostart-directory exists and if not
-        create it '''
-        autostart_dir = os.path.dirname(self.user_desktop_file)
-        if not os.path.exists(autostart_dir):
-            os.makedirs(autostart_dir)
-
         self.apply_settings()
         self.apply.set_sensitive(False)
 
@@ -198,61 +185,61 @@ class LightLockerSettings:
         Gtk.main_quit()
 
 # Process Management
-    if old_psutil_format:
-        def check_running_process(self, process_name):
-            """Return True if the specified process is active."""
-            # Find the process...
-            for pid in psutil.get_pid_list():
-                p = psutil.Process(pid)
-                if p.username == username:
-                    # When found, end the light-locker process.
-                    try:
-                        if os.path.basename(p.exe) == process_name:
-                            return True
-                    except psutil._error.AccessDenied:
-                        pass
-            return False
+    def get_process_username(self, process):
+        """Return the username of the process owner."""
+        p_user = None
 
-        def stop_light_locker(self):
-            """Safely stop the light-locker process."""
-            # Find the process...
-            for pid in psutil.get_pid_list():
-                p = psutil.Process(pid)
-                if p.username == username:
-                    # When found, end the light-locker process.
-                    try:
-                        if os.path.basename(p.exe) == 'light-locker':
-                            p.terminate()
-                    except psutil._error.AccessDenied:
-                        pass
+        try:
+            if old_psutil_format:
+                p_user = process.username
+            else:
+                p_user = process.username()
+        except:
+            pass
 
-    else:
-        def check_running_process(self, process_name):
-            """Return True if the specified process is active."""
-            # Find the process...
-            for pid in psutil.get_pid_list():
-                p = psutil.Process(pid)
-                if p.username() == username:
-                    # When found, end the light-locker process.
-                    try:
-                        if os.path.basename(p.exe()) == process_name:
-                            return True
-                    except psutil.AccessDenied:
-                        pass
-            return False
+        return p_user
 
-        def stop_light_locker(self):
-            """Safely stop the light-locker process."""
-            # Find the process...
-            for pid in psutil.get_pid_list():
+    def get_process_name(self, process):
+        """Return the name of the running process."""
+        p_name = None
+
+        try:
+            if old_psutil_format:
+                p_name = os.path.basename(process.exe)
+            else:
+                p_name = os.path.basename(process.exe())
+        except:
+            pass
+
+        return p_name
+
+    def check_running_process(self, process_name):
+        """Return True if the specified process is active."""
+        # Find the process...
+        for pid in psutil.get_pid_list():
+            try:
                 p = psutil.Process(pid)
-                if p.username() == username:
+                if self.get_process_username(p) == username:
+                    # Return True if the process is found.
+                    if self.get_process_name(p) == process_name:
+                        return True
+            except:
+                pass
+
+        return False
+
+    def stop_light_locker(self):
+        """Safely stop the light-locker process."""
+        # Find the process...
+        for pid in psutil.get_pid_list():
+            try:
+                p = psutil.Process(pid)
+                if self.get_process_username(p) == username:
                     # When found, end the light-locker process.
-                    try:
-                        if os.path.basename(p.exe()) == 'light-locker':
-                            p.terminate()
-                    except psutil.AccessDenied:
-                        pass
+                    if self.get_process_name(p) == 'light-locker':
+                        p.terminate()
+            except:
+                pass
 
     def run_command(self, cmd, check_output=False):
         '''Run a shell command, return its output.'''
@@ -270,6 +257,161 @@ class LightLockerSettings:
     def run_command_cb(self, widget, cmd):
         self.run_command(cmd, False)
 
+# Light Locker 1.5.1
+    def gsettings_init(self):
+        self.gsettings = None
+        schema_source = Gio.SettingsSchemaSource.get_default()
+        if (schema_source.lookup('apps.light-locker', False)):
+            self.gsettings = Gio.Settings.new('apps.light-locker')
+
+    def gsettings_available(self):
+        return self.gsettings is not None
+
+    def gsettings_get_settings(self):
+        lock_after_screensaver = self.gsettings.get_uint('lock-after-screensaver')
+        late_locking = self.gsettings.get_boolean('late-locking')
+        lock_on_suspend = self.gsettings.get_boolean('lock-on-suspend')
+
+        settings = dict()
+        settings['light-locker-enabled'] = self.get_light_locker_enabled()
+        settings['lock-after-screensaver'] = lock_after_screensaver > 0
+        settings['late-locking'] = late_locking
+        settings['lock-on-suspend'] = lock_on_suspend
+        settings['lock-time'] = \
+            self.light_locker_time_down_scaler(lock_after_screensaver)
+
+        return settings
+
+    def gsettings_set_enabled(self, enable):
+        if enable:
+            light_locker_exec = "light-locker"
+        else:
+            light_locker_exec = ""
+        keyfile = self.get_light_locker_autostart()
+        keyfile.set_value("Desktop Entry", "Exec", light_locker_exec)
+        self.save_light_locker_autostart()
+
+    def gsettings_set_late_locking(self, enable):
+        self.gsettings.set_boolean("late-locking", enable)
+
+    def gsettings_set_lock_after_screensaver(self, value):
+        self.gsettings.set_uint("lock-after-screensaver", value)
+
+    def gsettings_set_lock_on_suspend(self, enable):
+        self.gsettings.set_boolean("lock-on-suspend", enable)
+
+# Key Files
+    def ll_keyfile_get_settings(self):
+        # Defaults
+        settings = {
+            'light-locker-enabled': False,
+            'lock-after-screensaver': False,
+            'late-locking': False,
+            'lock-on-suspend': False,
+            'lock-time': 10
+        }
+
+        keyfile = self.get_light_locker_autostart()
+        if self.get_light_locker_enabled():
+            settings['light-locker-enabled'] = True
+
+            ll_exec = keyfile.get_value("Desktop Entry", "Exec");
+            value = ll_exec.replace("=", " ")
+            splitArgs = shlex.split(value)
+
+            parser = argparse.ArgumentParser(
+                description='Light Locker Settings')
+            parser.add_argument("--lock-after-screensaver")
+            parser.add_argument("--late-locking", action='store_true')
+            parser.add_argument("--lock-on-suspend", action='store_true')
+            (args, others) = parser.parse_known_args(splitArgs)
+
+            # Lock after screensaver
+            if args.lock_after_screensaver:
+                if int(args.lock_after_screensaver) != 0:
+                    settings['lock-after-screensaver'] = True
+                    settings['lock-time'] = self.light_locker_time_down_scaler(
+                        int(args.lock_after_screensaver))
+
+            # Late Locking
+            if args.late_locking:
+                settings['late-locking'] = True
+
+            # Lock on Suspend
+            if args.lock_on_suspend:
+                settings['lock-on-suspend'] = True
+
+        return settings
+
+    def get_autostart(self, filename, defaults={}):
+        autostart = os.path.join(GLib.get_user_config_dir(), 'autostart')
+        if not os.path.exists(autostart):
+            os.makedirs(autostart)
+        keyfile = GLib.KeyFile.new()
+
+        dirs = []
+        dirs.append (autostart)
+        for directory in (GLib.get_system_config_dirs()):
+            dirs.append (os.path.join(directory, 'autostart'))
+
+        keyfile.load_from_dirs(filename, dirs,
+                               GLib.KeyFileFlags.KEEP_TRANSLATIONS)
+
+        for key in defaults.keys():
+            try:
+                if keyfile.get_value("Desktop Entry", key) is None:
+                    keyfile.set_value("Desktop Entry", key, defaults[key])
+            except GLib.Error:
+                keyfile.set_value("Desktop Entry", key, defaults[key])
+
+        return keyfile
+
+    def get_light_locker_autostart(self):
+        if self.light_locker_keyfile is not None:
+            return self.light_locker_keyfile
+
+        defaults = {
+            "Type": "Application",
+            "Name": _("Screen Locker"),
+            "Comment": _("Launch screen locker program"),
+            "Icon": "preferences-desktop-screensaver",
+            "NoDisplay": "true",
+            "NotShownIn": "Gnome;Unity",
+            "Exec": ""
+        }
+
+        self.light_locker_keyfile = \
+            self.get_autostart('light-locker.desktop', defaults)
+
+        return self.light_locker_keyfile
+
+    def get_screensaver_autostart(self):
+        if self.screensaver_keyfile is not None:
+            return self.screensaver_keyfile
+
+        defaults = {
+            "Type": "Application",
+            "Name": _("Screensaver"),
+            "Comment": _("Set screensaver timeouts"),
+            "Exec": ""
+        }
+
+        self.screensaver_keyfile = \
+            self.get_autostart('screensaver-settings.desktop', defaults)
+        return self.screensaver_keyfile
+
+    def save_light_locker_autostart(self):
+        filename = os.path.join(GLib.get_user_config_dir(), 'autostart',
+                                'light-locker.desktop')
+        autostart = self.get_light_locker_autostart()
+        autostart.save_to_file(filename)
+
+    def save_screensaver_autostart(self):
+        filename = os.path.join(GLib.get_user_config_dir(), 'autostart',
+                                'screensaver-settings.desktop')
+        autostart = self.get_screensaver_autostart()
+        autostart.save_to_file(filename)
+
 # Settings Parsing
     def use_screensaver_manager(self, name, command):
         """Replace the Screensaver settings with a different application."""
@@ -281,8 +423,10 @@ class LightLockerSettings:
         screensaver_frame = self.builder.get_object("screensaver_details")
 
         # Light Locker Settings is *NOT* controlling the screensaver.
-        if os.path.isfile(self.screenblank_timeout_desktop_file):
-            os.remove(self.screenblank_timeout_desktop_file)
+        filename = os.path.join(GLib.get_user_config_dir(), 'autostart',
+                                'screensaver-settings.desktop')
+        if os.path.isfile(filename):
+            os.remove(filename)
         screensaver_frame.hide()
 
         # Update the InfoBar
@@ -292,30 +436,24 @@ class LightLockerSettings:
         infobar.show()
 
     def init_settings(self):
-        # Get the Settings, first from light-locker, then override as needed
-        ''' if there is a file exists in ~/.config/autostart/ parse that
-            for current settings picked by the user else load in the
-            systemwide '''
-        if os.path.isfile(self.user_desktop_file):
-            desktop_file = self.user_desktop_file
-        elif os.path.isfile(self.systemwide_desktop_file):
-            desktop_file = self.systemwide_desktop_file
+        if self.gsettings_available():
+            settings = self.gsettings_get_settings()
+            ll_exec_settings = self.ll_keyfile_get_settings()
+            if ll_exec_settings['lock-after-screensaver']:
+                settings['lock-after-screensaver'] = True
+            if ll_exec_settings['late-locking']:
+                settings['late-locking'] = True
+            if ll_exec_settings['lock-on-suspend']:
+                settings['lock-on-suspend'] = True
+            if ll_exec_settings['lock-time'] != 10:
+                settings['lock-time'] = ll_exec_settings['lock-time']
         else:
-            desktop_file = None
-
-        settings = self.read_in_desktop(desktop_file)
-        use_light_locker = settings['light-locker-enabled']
-        lock_after_screensaver = settings['lock-after-screensaver']
-        late_locking = settings['late-locking']
-        lock_on_suspend = settings['lock-on-suspend']
-        lock_time = settings['lock-time']
-        screen_blank_timeout, screen_off_timeout = \
-            self.get_screen_blank_timeout()
+            settings = self.ll_keyfile_get_settings()
 
         # Replace settings with xfce4-power-manager
         if self.check_running_process("xfce4-power-manager"):
             xfpm_sync = light_locker_xfsync.XfpmSync()
-            lock_on_suspend = xfpm_sync.get_lock()
+            settings['lock-on-suspend'] = xfpm_sync.get_lock()
 
         # Check if any known screensaver managers are currently running.
         for process_name in screensaver_managers.keys():
@@ -324,6 +462,15 @@ class LightLockerSettings:
                 self.use_screensaver_manager(name, command)
                 break
 
+        # Extract the settings
+        use_light_locker = settings['light-locker-enabled']
+        lock_after_screensaver = settings['lock-after-screensaver']
+        late_locking = settings['late-locking']
+        lock_on_suspend = settings['lock-on-suspend']
+        lock_time = settings['lock-time']
+        screen_blank_timeout, screen_off_timeout = \
+            self.get_screen_blank_timeout()
+
         # Apply the settings
         self.use_lightlocker.set_active(use_light_locker)
         self.session_lock_combo.set_sensitive(use_light_locker)
@@ -348,75 +495,10 @@ class LightLockerSettings:
         self.screenblank_timeout.set_value(blank)
         self.screenoff_timeout.set_value(off)
 
-    def read_in_desktop(self, filepath):
-        '''Parse the .desktop file for light-locker settings.'''
-        if filepath is not None:
-            try:
-                with open(filepath) as f:
-                    self.content = f.readlines()
-                    for line in self.content:
-                        if line.startswith("Exec="):
-                            return self.parse_options(line)
-            except:
-                pass
-
-        # Fallback if something fails.
-        self.content = "[Desktop Entry]\n" \
-                       "Type=Application\n" \
-                       "Name=Screen Locker\n" \
-                       "Comment=Launch screen locker program\n" \
-                       "Icon=preferences-desktop-screensaver\n" \
-                       "Exec=\n" \
-                       "NoDisplay=true"
-        return self.parse_options("Exec=\n")
-
-    def parse_options(self, options_line):
-        '''Parse command-line arguments from the Exec line.'''
-        # Defaults
-        use_light_locker = False
-        lock_after_screensaver = False
-        late_locking = False
-        lock_on_suspend = False
-        lock_time = 10
-
-        # Get the settings from the configuration file.
-        if "light-locker" in options_line:
-            use_light_locker = True
-
-            ''' replace the = so we can parse it easily '''
-            options_line = options_line.replace("=", " ")
-            splitArgs = shlex.split(options_line)
-
-            parser = argparse.ArgumentParser(
-                description='Light Locker Settings')
-            parser.add_argument("--lock-after-screensaver")
-            parser.add_argument("--late-locking", action='store_true')
-            parser.add_argument("--lock-on-suspend", action='store_true')
-            (args, others) = parser.parse_known_args(splitArgs)
-
-            # Lock after screensaver
-            if args.lock_after_screensaver:
-                if int(args.lock_after_screensaver) != 0:
-                    lock_after_screensaver = True
-                    lock_time = self.light_locker_time_down_scaler(
-                        int(args.lock_after_screensaver))
-
-            # Late Locking
-            if args.late_locking:
-                late_locking = True
-
-            # Lock on Suspend
-            if args.lock_on_suspend:
-                lock_on_suspend = True
-
-        settings = dict()
-        settings['light-locker-enabled'] = use_light_locker
-        settings['lock-after-screensaver'] = lock_after_screensaver
-        settings['late-locking'] = late_locking
-        settings['lock-on-suspend'] = lock_on_suspend
-        settings['lock-time'] = lock_time
-
-        return settings
+    def get_light_locker_enabled(self):
+        keyfile = self.get_light_locker_autostart()
+        ll_exec = keyfile.get_value("Desktop Entry", "Exec")
+        return "light-locker" in ll_exec
 
     def get_screen_blank_timeout(self):
         ''' read in the X11 screensaver settings from bash '''
@@ -548,14 +630,23 @@ class LightLockerSettings:
 
     def apply_light_locker_settings(self, settings):
         '''Apply the light-locker settings'''
-        # Stop any running light-locker processes.
-        self.stop_light_locker()
-
         lock_enabled = settings['lock-enabled']
         late_locking = settings['late-locking']
         lock_on_suspend = settings['lock-on-suspend']
         lock_after_screensaver = settings['lock-after-screensaver']
 
+        # If GSettings is available, prefer the following method.
+        if self.gsettings_available():
+            self.gsettings_set_enabled(lock_enabled)
+            self.gsettings_set_late_locking(late_locking)
+            self.gsettings_set_lock_after_screensaver(lock_after_screensaver)
+            self.gsettings_set_lock_on_suspend(lock_on_suspend)
+            return
+
+        # Else, proceed with the legacy code below
+        # Stop any running light-locker processes.
+        self.stop_light_locker()
+
         if late_locking:
             late_locking = "--late-locking"
         else:
@@ -577,7 +668,9 @@ class LightLockerSettings:
                                            lock_on_suspend, late_locking)
 
         # Save the light-locker autostart file.
-        self.write_light_locker_file(self.user_desktop_file, light_locker_exec)
+        keyfile = self.get_light_locker_autostart()
+        keyfile.set_value("Desktop Entry", "Exec", light_locker_exec)
+        self.save_light_locker_autostart()
 
         # Execute the updated light-locker command.
         self.run_command(light_locker_exec)
@@ -595,29 +688,9 @@ class LightLockerSettings:
         self.run_command(screensaver_exec)
 
         # Save the screensaver autostart file.
-        self.write_screensaver_file(
-            self.screenblank_timeout_desktop_file, screensaver_exec)
-
-    def write_light_locker_file(self, filepath, light_locker_command):
-        '''Write the updated light-locker settings to its .desktop file.'''
-        exec_str = "Exec=%s\n" % light_locker_command
-        with open(filepath, 'w') as f:
-            for line in self.content:
-                if line.startswith("Exec="):
-                    f.write(exec_str)
-                else:
-                    f.write(line)
-
-    def write_screensaver_file(self, filepath, screensaver_command):
-        '''Write the updated screensaver settings to its .desktop file.'''
-        with open(filepath, 'w') as f:
-            f.write(
-                "[Desktop Entry]\n"
-                "Name=%s\n"
-                "Comment=%s\n"
-                "Exec=%s\n"
-                % (_("Screensaver"), _("Set screensaver timeouts"),
-                    screensaver_command))
+        keyfile = self.get_screensaver_autostart()
+        keyfile.set_value("Desktop Entry", "Exec", screensaver_exec)
+        self.save_screensaver_autostart()
 
 
 if __name__ == "__main__":
